@@ -1,4 +1,4 @@
-# Copyright (c) 2012-2015, Eucalyptus Systems, Inc.
+# Copyright (c) 2012-2016 Hewlett Packard Enterprise Development LP
 #
 # Permission to use, copy, modify, and/or distribute this software for
 # any purpose with or without fee is hereby granted, provided that the
@@ -32,7 +32,7 @@ from requestbuilder.xmlparse import parse_listdelimited_aws_xml
 
 
 class BaseRequest(BaseCommand):
-    '''
+    """
     The basis for a command line tool that represents a request.  The data for
     this request are stored in a few instance members:
      - method:   the HTTP method to use (e.g. 'GET').  Defaults to self.METHOD.
@@ -60,10 +60,12 @@ class BaseRequest(BaseCommand):
     Important members of this class, in addition to those inherited from
     BaseCommand, include:
      - SERVICE_CLASS:  a class corresponding to the web service in use
-     - NAME:           a string representing the name of this request.  This
-                       defaults to the class's name.
+     - AUTH_CLASS:     a class corresponding to the authentication method
+                       to use, if any
+     - NAME:           a string representing the name of this request.
+                       This defaults to the class's name.
      - METHOD:         the HTTP method to use by default
-    '''
+    """
 
     SERVICE_CLASS = BaseService
     AUTH_CLASS = None
@@ -73,8 +75,6 @@ class BaseRequest(BaseCommand):
     DEFAULT_ROUTES = (PARAMS,)
 
     def __init__(self, service=None, auth=None, **kwargs):
-        self.auth = auth
-        self.service = service
         # Parts of the HTTP request to be sent to the server.
         self.method = self.METHOD
         self.path = None
@@ -87,16 +87,25 @@ class BaseRequest(BaseCommand):
         self.response = None
 
         self.__configured = False
+        self.__auth = auth
+        self.__service = service
 
         BaseCommand.__init__(self, **kwargs)
 
-    def _post_init(self):
-        if self.service is None and self.SERVICE_CLASS is not None:
-            self.service = self.SERVICE_CLASS(self.config,
-                                              loglevel=self.log.level)
-        if self.auth is None and self.AUTH_CLASS is not None:
-            self.auth = self.AUTH_CLASS(self.config, loglevel=self.log.level)
-        BaseCommand._post_init(self)
+    @property
+    def auth(self):
+        if not self.__auth and self.AUTH_CLASS is not None:
+            # pylint: disable=not-callable
+            self.__auth = self.AUTH_CLASS(self.config, loglevel=self.log.level)
+            # pylint: enable=not-callable
+        return self.__auth
+
+    @property
+    def service(self):
+        if not self.__service and self.SERVICE_CLASS is not None:
+            self.__service = self.SERVICE_CLASS(self.config,
+                                                loglevel=self.log.level)
+        return self.__service
 
     @classmethod
     def from_other(cls, other, **kwargs):
@@ -125,6 +134,7 @@ class BaseRequest(BaseCommand):
             self.auth.args.update(self.args)
 
     def configure(self):
+        # TODO:  rename this to setup
         BaseCommand.configure(self)
         if self.service is not None:
             self.service.configure()
@@ -136,16 +146,6 @@ class BaseRequest(BaseCommand):
     def name(self):
         return self.NAME or self.__class__.__name__
 
-    @property
-    def status(self):
-        msg = 'BaseRequest.status is deprecated, use response status'
-        self.log.warn(msg)
-        warnings.warn(msg, DeprecationWarning)  # deprecated in 0.3
-        if self.response is not None:
-            return self.response.status
-        else:
-            return None
-
     def send(self):
         if not self.__configured:
             self.log.warn('send() called before configure(); bugs may result')
@@ -266,11 +266,11 @@ class AWSQueryRequest(BaseRequest):
         self.log.info('parameters: %s', redacted_params)
 
         if self.method.upper() == 'POST':
-            self.log.debug('sending flattened params as form data')
+            self.log.debug('sending flattened parameters as form data')
             self.body = params
             self.params = {}
         else:
-            self.log.debug('sending flattened params as query string')
+            self.log.debug('sending flattened parameters as query string')
             self.params = params
         try:
             return BaseRequest.send(self)
@@ -336,6 +336,8 @@ class AWSQueryRequest(BaseRequest):
                     flattened[prefixed_key] = str(val).lower()
                 elif isinstance(val, file):
                     flattened[prefixed_key] = val.read()
+                elif isinstance(val, float):
+                    flattened[prefixed_key] = str(val)
                 elif val or val is 0:
                     flattened[prefixed_key] = str(val)
                 elif val is EMPTY:
@@ -375,7 +377,9 @@ class AWSQueryRequest(BaseRequest):
 
         helplines = ['allowed filter names:']
         for filter_obj in self.FILTERS:
-            if filter_obj.help:
+            if filter_obj.help == argparse.SUPPRESS:
+                continue
+            elif filter_obj.help:
                 first, _, rest = filter_obj.help.partition('\n')
                 if rest.startswith(' ') and not first.startswith(' '):
                     # First line is not uniformly indented
