@@ -1,4 +1,4 @@
-# Copyright (c) 2012-2015, Eucalyptus Systems, Inc.
+# Copyright (c) 2012-2016 Hewlett Packard Enterprise Development LP
 #
 # Permission to use, copy, modify, and/or distribute this software for
 # any purpose with or without fee is hereby granted, provided that the
@@ -22,7 +22,6 @@ import io
 import logging
 import os.path
 import random
-import socket
 import time
 
 import requests.exceptions
@@ -30,7 +29,7 @@ import six
 import six.moves.urllib_parse as urlparse
 
 from requestbuilder.exceptions import (ClientError, ServerError,
-                                       ServiceInitError)
+                                       ServiceInitError, TimeoutError)
 from requestbuilder.mixins import RegionConfigurableMixin
 
 
@@ -73,6 +72,8 @@ class BaseService(RegionConfigurableMixin):
         return new
 
     def configure(self):
+        # TODO:  rename this to setup
+        #
         # Configure user and region before grabbing endpoint info since
         # the latter may depend upon the former
         self.update_config_view()
@@ -92,12 +93,11 @@ class BaseService(RegionConfigurableMixin):
             else:
                 self.timeout = self.TIMEOUT
 
-        # SSL cert verification is opt-in
-        self.session_args['verify'] = self.config.convert_to_bool(
-            self.config.get_region_option('verify-ssl'), default=False)
+        self.session_args.setdefault('stream', True)
 
-        # requests only applies proxy config in code paths we don't use
-        self.session_args['proxies'] = _get_proxies()
+        # SSL cert verification is opt-in
+        self.session_args.setdefault('verify', self.config.convert_to_bool(
+            self.config.get_region_option('verify-ssl'), default=False))
 
         # Ensure everything is okay and finish up
         self.validate_config()
@@ -108,6 +108,9 @@ class BaseService(RegionConfigurableMixin):
             self._session = requests.session()
             for key, val in self.session_args.iteritems():
                 setattr(self._session, key, val)
+            for adapter in self._session.adapters.values():
+                # send_request handles retries to allow for re-signing
+                adapter.max_retries = 0
         return self._session
 
     def validate_config(self):
@@ -169,8 +172,28 @@ class BaseService(RegionConfigurableMixin):
                     p_request = self.__log_and_prepare_request(
                         method, url, params, data, files, headers, auth)
                     p_request.start_time = datetime.datetime.now()
-                    response = self.session.send(p_request, stream=True,
-                                                 timeout=self.timeout)
+                    proxies = requests.utils.get_environ_proxies(url)
+                    for key, val in sorted(proxies.items()):
+                        self.log.debug('request proxy:  %s=%s', key, val)
+                    try:
+                        response = self.session.send(
+                            p_request, timeout=self.timeout, proxies=proxies,
+                            allow_redirects=False)
+                    except requests.exceptions.Timeout:
+                        if attempt_no < max_tries:
+                            self.log.debug('timeout', exc_info=True)
+                            if data_file_offset is not None:
+                                self.log.debug('re-seeking body to '
+                                               'beginning of file')
+                                # pylint: disable=E1101
+                                data.seek(data_file_offset)
+                                # pylint: enable=E1101
+                                continue
+                            elif not hasattr(data, 'tell'):
+                                continue
+                            # Fallthrough -- if it has a file pointer but not
+                            # seek we can't retry because we can't rewind.
+                        raise
                     if response.status_code not in (500, 503):
                         break
                     # If it *was* in that list, retry
@@ -203,6 +226,9 @@ class BaseService(RegionConfigurableMixin):
                     # redirect another way for some reason.
                     self.handle_http_error(response)
                 return response
+        except requests.exceptions.Timeout as exc:
+            self.log.debug('timeout', exc_info=True)
+            raise TimeoutError('request timed out', exc)
         except requests.exceptions.ConnectionError as exc:
             self.log.debug('connection error', exc_info=True)
             return self.__handle_connection_error(exc)
@@ -220,7 +246,8 @@ class BaseService(RegionConfigurableMixin):
                 msg = err.args[0].reason
             elif isinstance(err.args[0], Exception):
                 return self.__handle_connection_error(err.args[0])
-            msg = err.args[0]
+            else:
+                msg = err.args[0]
         else:
             raise ClientError('connection error')
         raise ClientError('connection error ({0})'.format(msg))
@@ -250,7 +277,7 @@ class BaseService(RegionConfigurableMixin):
         request = requests.Request(method=method, url=url, params=params,
                                    data=data, files=files, headers=headers,
                                    auth=bound_auth)
-        p_request = request.prepare()
+        p_request = self.session.prepare_request(request)
         p_request.hooks = {'response': hooks['response']}
         self.log.debug('request method: %s', request.method)
         self.log.debug('request url:    %s', p_request.url)
@@ -261,7 +288,8 @@ class BaseService(RegionConfigurableMixin):
                 self.log.debug('request header: %s: %s', key, val)
         if isinstance(request.params, (dict, collections.Mapping)):
             for key, val in sorted(urlparse.parse_qsl(
-                    urlparse.urlparse(p_request.url).query)):
+                    urlparse.urlparse(p_request.url).query,
+                    keep_blank_values=True)):
                 if key.lower().endswith('password'):
                     val = '<redacted>'
                 self.log.debug('request param:  %s: %s', key, val)
@@ -272,14 +300,17 @@ class BaseService(RegionConfigurableMixin):
                 data = cgi.parse_multipart(io.BytesIO(p_request.body),
                                            content_type_params)
             elif content_type == 'application/x-www-form-urlencoded':
-                data = dict(urlparse.parse_qsl(p_request.body))
+                data = dict(urlparse.parse_qsl(p_request.body,
+                                               keep_blank_values=True))
             else:
                 data = request.data
             for key, val in sorted(data.items()):
+                # pylint: disable=superfluous-parens
                 if key in (request.files or {}):
                     # We probably don't want to include the contents of
                     # entire files in debug output.
                     continue
+                # pylint: enable=superfluous-parens
                 if key.lower().endswith('password'):
                     val = '<redacted>'
                 self.log.debug('request data:   %s: %s', key, val)
@@ -347,14 +378,3 @@ def _parse_endpoint_url(urlish):
         region = None
         url = urlish
     return url, region
-
-
-def _get_proxies():
-    try:
-        bypass = six.moves.urllib.request.proxy_bypass()
-    except (TypeError, socket.gaierror):
-        # This blows up on my old OS X machine
-        bypass = False
-    if bypass:
-        return {}
-    return six.moves.urllib.request.getproxies()
